#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Thu Aug  1 14:37:04 2019

@author: jtu
"""
import math
import numpy as np
import Cs_Rind_Lt
from scipy import interpolate
import matplotlib.pyplot as plt

def calc_Q(Va,freq):
    #performing interpolate to increase frequency resolution so as to increase accuracy
    #of the Q calculation
    maxfq=max(freq)
    fqin=np.linspace(min(freq), maxfq, len(freq)*30)
    fq=np.flipud(fqin)

    Va_interp=interpolate.interp1d(freq, Va, kind='linear')
    Va1=Va_interp(fq)

    ipeak=np.argmax(Va1)
    V70p=np.max(Va1)/np.sqrt(2)

    iL1=-1
    iL=-1
    iL2=-1
    #original order is from high to low frequency - needed to be reversed
    for i in range(ipeak, len(fq)):
        if Va1[i] > V70p:                       #  +            ipeak
            iL2=i                              #   +        
        elif Va1[i] == V70p:                    #     +         iL2
            iL=i                               #       +    ------- V70p (iL)
            break                              #         +     iL1
        else:                                  #          +
            iL1=i                              #            +  len(freq)
            break

    for i in range(ipeak, iL1):
        if Va1[i+1] >= Va1[i]:
            return -2.0                       #Va non monotonous dcrease

    if iL == -1:
        if iL2 >= ipeak and iL1 > iL2:
            fhL= (fq[iL1]*(Va1[iL2]-V70p)+fq[iL2]*(V70p-Va1[iL1])) \
                /(Va1[iL2]-Va1[iL1])
        else:
            return -1.0
    else:
        fhL=fq[iL]

    iH1=-1
    iH=-1
    iH2=-1
    for i in range(0, ipeak+1):
        if Va1[i] < V70p:                       #           +  ipeak
            iH1=i                              #          +
        elif Va1[i] == V70p:                    #        +     iH2
            iH=i                               #       +   --------V70p (iH)
            break                              #      +       iH1
        else:                                  #    +
            iH2=i                              #   +          0
            break

    for i in range(iH1,ipeak):
        if Va1[i+1] <= Va1[i]:
            return -2.0                       #Va non monotonous increase

    if iH == -1:
        if iH1 >= 0 and iH2 > iH1:
            fhH= (fq[iH1]*(Va1[iH2]-V70p)+fq[iH2]*(V70p-Va1[iH1])) \
                /(Va1[iH2]-Va1[iH1])
        else:
            return -1.0
    else:
        fhH=fq[iH]

    #fhL=np.interp(V70p, Va[0:ipeak], freq[0:ipeak])
    #Vrev=np.zeros(imax-ipeak+1)
    #frev=np.zeros(imax-ipeak+1)
    #for i in range(0, imax-ipeak+1):
    #    Vrev[i]=Va[imax-i]
    #    frev[i]=freq[imax-i]
    #fhH=np.interp(V70p, Vrev, frev)

    #Q=fq[ipeak]/(fhH-fhL)
    Q=freq[np.argmax(Va)]/(fhH-fhL)

    #if k==2000:
    #    print(Q, fhH, fhL)
        # Plot the result
        #plt.figure(figsize=(8,6))
        #plt.plot(fq, Va1, 'red', label='Interpolated')
        #plt.scatter(fq, Va1, color='black', marker=".", s=20, edgecolors="none",  \
        #    label='Interpolated')
        #plt.scatter(freq, Va, color='red', marker=".", s=60, edgecolors="none", \
        #    label='Original')
        #plt.legend(loc='upper left', frameon=False)
        #plt.xlabel('Frequency (Hz)')
        #plt.ylabel('Antenna Voltage (V)')
        #plt.show()

    return Q

def Ca_Ra_symmetric(entv, entv3, Q, Va1, phi1, Va2, phi2, freq):
    pi=math.pi
    pi2=2.0*pi
    #rad=pi/180.0

    nfsp=np.argmax(Va1)
    nfsm=np.argmax(Va2)

    #phip=phi1[nfsp]*rad
    #phim=phi2[nfsm]*rad

    Cs1p, R1p, Ltp=Cs_Rind_Lt.Cs_RL_Lt(entv[1], freq[nfsp])
    Cs1m, R1m, Ltm=Cs_Rind_Lt.Cs_RL_Lt(entv[4], freq[nfsm])
    Cip=entv[2]*1.0e-12+Cs1p
    Cim=entv[5]*1.0e-12+Cs1m

    #see note Resonance_solutionOf-Ra&Ca.docx
    #for entire dipole
    Ci_pri=Cip*Cim/(Cip+Cim)

    Omegar=pi2*freq[nfsp]
    XL=Omegar*(Ltp+Ltm)
    XC=1.0/(Omegar*Ci_pri)

    Rp=XL/Q[0]-(R1p+R1m)

    temp=(XC-XL)**2+Rp*Rp
    Rap=Rp*XC*XC/temp
    Cap=Ci_pri*temp/((XC-XL)*XL-Rp)

    Iap=Va1[nfsp]/np.sqrt(Rap*Rap+1.0/(Omegar*Cap)**2)

    #compare Itot calculated using V_source/Rtot and Va/Z_1: 
    # Z_1=R_1+i X_1 where R_1 = Rtot-R_int and X_1 = X_L at resonance
    #Itot=Va1[nfsp]/np.sqrt(Rp*Rp+XL*XL)
    #print(entv[0], Itot/(entv[0]*Q[0]/XL))

    #for TATU minus
    Omegar=pi2*freq[nfsm]
    XL=Omegar*Ltm
    XC=1.0/(Omegar*Cim)

    Rp=XL/Q[1]-R1m
  
    temp=(XC-XL)**2+Rp*Rp
    Ram=Rp*XC*XC/temp

    Cam=Cim*temp/((XC-XL)*XC-temp)

    Iam=Va2[nfsm]/np.sqrt(Ram*Ram+1.0/(Omegar*Cam)**2)

    #current divided by a factor of sqrt(2), then close to measured current
    return Rap, Ram, Cap*1.0e12, Cam*1.0e12, Iap, Iam

def Ca_Ra_asymmetr(entv, entv3, Q, Va1, phi1, Va2, phi2, freq):
    pi=math.pi
    pi2=2.0*pi
    #rad=pi/180.0

    nfsp=np.argmax(Va1)
    nfsm=np.argmax(Va2)

    #phip=phi1[nfsp]*rad
    #phim=phi2[nfsm]*rad

    #see note Resonance_solutionOf-Ra&Ca.docx & and equivalent circuit for assymetric case
    #TATU minus has only a Cstray
    Csp, RLp, Ltp=Cs_Rind_Lt.Cs_RL_Lt(entv[1], freq[nfsp])
    Csm =125.2697113202511e-12 #, _, _=Cs_Rind_Lt.Cs_RL_Lt(entv[4], freq[nfsm])

    Cip=entv[2]*1.0e-12+Csp

    Omegar=pi2*freq[nfsp]
    XL=Omegar*Ltp
    XC=1.0/(Omegar*Cip)
    R1=XL/Q[0]-RLp

    temp=(XC-XL)**2+R1*R1
    Rap=XC*XC*R1/temp
    #Cb=Cip*temp/((XC-XL)*XL-R1*R1)

    # calculated Ca by Ia = Va1/|Ra-i/(Omegar*Ca)-i/(Omegar*Csm)| = Va2/|1/(Omegar*Csm)|
    Xcb=np.sqrt((Va1[nfsp]/(Va2[nfsm]*Omegar*Csm))**2-Rap*Rap)
    Cap=1.0/(Xcb*Omegar-1.0/Csm)

    #resonance condition with assumption (Omwgar*(C1+Csp)*Ra)**2 << (1+(C1+Csp)/Ca)**2
    #Ctot=1.0/(Omegar*Omegar*Ltp)
    #Cb=Ctot-Cip
    #Cap=1.0/(1.0/Cb-1.0/Csm)
    #Xcb=1.0/(Omegar*Cap)+1.0/(Omegar*Csm)
    #Rap=np.sqrt(Va1[nfsp]*Va1[nfsp]/Iap**2-Xcb*Xcb)

    Iap=Va2[nfsm]*Omegar*Csm

    #Here Rap and Cap are capacitance and resistance of two monopole connected in seriers
    # The current is that flows from Y+ to Y-. 
    #Cap=(Csm*Cb)/(Csm-Cb)
    #Iap=Va1[nfsp]/np.sqrt(Rap*Rap+1.0/(Omegar*Cb)**2)

    #compare Itot calculated using V_source/Rtot and Va/Z_1: 
    # Z_1=R_1+i X_1 where R_1 = Rtot-R_int and X_1 = X_L at resonance
    #Itot=Va1[nfsp]/np.sqrt(Rp*Rp+XL*XL)
    #print(entv[0], Itot/(entv[0]*Q[0]/XL))

    #for TATU minus
    # The resistance and capacitance of one branch is halfed and doubled
    #Rap=Rap/2.0
    Ram=Rap/2.0
    Cam=Cap*2.0

    Iam=Iap

    #current divided by a factor of sqrt(2), then close to measured current
    return Rap, Ram, Cap*1.0e12, Cam*1.0e12, Iap, Iam

def Ca_Ra_comb(entv, entv3, Va1, phi1, Va2, phi2, Va31, phi31, Va32, phi32, freq):
    pi=math.pi
    pi2=2.0*pi
    rad=pi/180.0

    nfq=len(freq)
    Vat=np.zeros(nfq)

    for i in range(nfq):
        phip=phi1[i]*rad
        phim=phi2[i]*rad

        cosp=np.cos(phip)
        sinp=np.sin(phip)
        cosm=np.cos(phim)
        sinm=np.sin(phim)

        Vat[i]=np.sqrt((Va1[i]*cosp-Va2[i]*cosm)**2+(Va1[i]*sinp-Va2[i]*sinm)**2)
        
    Q=calc_Q(Vat,freq)
    if Q <= 0.0:
        return -1, -1, Vat, -1, -1

    nfs=np.argmax(Vat)

    Cs1p, R1p, Ltp=Cs_Rind_Lt.Cs_RL_Lt(entv[1], freq[nfs])
    Cs1m, R1m, Ltm=Cs_Rind_Lt.Cs_RL_Lt(entv[4], freq[nfs])
    C1p=entv[2]+Cs1p*1.0e12
    C1m=entv[5]+Cs1m*1.0e12

    Lt=Ltp+Ltm
    Cint=C1p*C1m/(C1p+C1m)
    Rint=R1p+R1m

    Omegar=pi2*freq[nfs]
    XL=Omegar*Lt

    Ca=1.0e12/(Omegar**2*Lt)-Cint

    Rp=XL/Q-Rint
    Ra=Rp*((Cint+Ca)/Ca)**2

    Ia=Vat[nfs]/np.abs(complex(Ra,-1.0e12/(Omegar*Ca)))

    return Ra, Ca, Vat, Ia, Q

def Ca_Ra1(entv, entv3, Q, Va1, phi1, Va2, phi2, Va31, phi31, Va32, phi32, freq):
    pi=math.pi
    pi2=2.0*pi
    rad=pi/180.0

    nfsp=np.argmax(Va1)
    nfsm=np.argmax(Va2)

    Cs1p, R1p, Ltp=Cs_Rind_Lt.Cs_RL_Lt(entv[1], freq[nfsp])
    Cs1m, R1m, Ltm=Cs_Rind_Lt.Cs_RL_Lt(entv[4], freq[nfsm])
    C1p=entv[2]*1.0e-12+Cs1p
    C1m=entv[5]*1.0e-12+Cs1m

    Omega=pi2*freq[nfsp]

    Lt=Ltp+Ltm
    Ct=1.0/(Omega**2*Lt)
    Ca=Ct-C1p*C1m/(C1p+C1m)

    Ya=1.0/(Omega*Ca)
    Rt=Omega*Lt/Q[0]

    Ra=(Rt-(R1p+R1m))*(Ct/Ca)**2

    phip=phi1[nfsp]*rad
    phim=phi2[nfsm]*rad
    Vapr=Va1[nfsp]*np.cos(phip)
    Vapi=Va1[nfsp]*np.sin(phip)
    Vamr=Va2[nfsm]*np.cos(phim)
    Vami=Va2[nfsm]*np.sin(phim)

    Ia=complex(Vapr-Vamr,Vapi-Vami)/complex(Ra,-Ya)

    return Ra, Ca*1.0e12, Ia
