#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Wed Sep 18 13:20:06 2019

@author: jtu
"""
import os
import numpy as np
import math
from scipy import stats

try:
    from matplotlib.backends.backend_tkagg import FigureCanvasTkAgg as figtk
except ImportError:
    from matplotlib.backends.backend_tkagg import FigureCanvasTk as figtk

try:
    from matplotlib.backends.backend_tkagg import NavigationToolbar2TkAgg as navitk
except ImportError:
    from matplotlib.backends.backend_tkagg import NavigationToolbar2Tk as navitk

from mpl_toolkits.axes_grid1.inset_locator import inset_axes

#implement th edefault matplotlib key bindings
import matplotlib.ticker as ticker
from matplotlib.backend_bases import key_press_handler
from matplotlib.figure import Figure
from matplotlib.ticker import MultipleLocator

import matplotlib.pyplot as plt
from matplotlib.ticker import AutoMinorLocator

import ant_impedance
import Cs_Rind_Lt
import days_of_year
import binned_values
import moving_average
import ant_impedance_v2

#####################################################################################
#####################################################################################
def plot_VaRaCaPrad(wd, canv, fig, toolbar, fpath, data_all, bwd, bname, inpflag, symm):

    print('Processing high power transmission data ...')

    fdir=fpath.strip()
    inpflag=inpflag.strip()
    if fdir[len(fdir)-1] == '/':
        logfile=fdir+bname+'_'+inpflag+'_excluded.dat'
        datafile=fdir+bname+'_'+inpflag+'_data.dat'
        errfile=fdir+bname+'_'+inpflag+'_excluded.log'
    else:
        logfile=fdir+'/'+bname+'_'+inpflag+'_excluded.dat'
        datafile=fdir+'/'+bname+'_'+inpflag+'_data.dat'
        errfile=fdir+'/'+bname+'_'+inpflag+'_excluded.log'

    s=('{:>7}{:>10}'+'{:>6}'*3+'{:>8}{:>6}'+'{:>8}'*2+'{:>5}'+'{:>6}'*2 \
        +'{:>8}{:>6}'+'{:>8}'*2+'{:>8}').format('date','time','V1','L1', \
        'C1','fres','C3','maxVa','minVa','V1','L1','C1','fres','C3','maxVa', \
        'minVa','prog#\n')

    logf=open(logfile,'w')
    logf.write('{:>42}{:>48}'.format('Y+','Y-\n'))
    logf.write(s)

    sd='------------------------------------'
    s=(('{:^10}{:^10}{:^5}'+'{:^7}'*3+'{:>4}{:>7}{:>11}{:>7}{:>8}'+'{:>7}'*2 \
        +'{:>5}{:>6}{:>7}{:>8}{:>6}{:>7}{:>11}{:>7}'+'{:>7}'*3+'{:>7}{:>6}'
        +'{:>9}{:>11}'+'{:>8}{:>7}').format('date','time','V1','L1', \
        'C1','fres','Va','Q','Ia(mA)','Ra','RL','Ca','Cs','V1','L1','C1','fres', \
        'Va','Q','Ia(mA)','Ra','RL','Ca', 'Cs','InOut','Prog#','Phase1','Phase2', \
        'power+','power-\n'))

    dataf=open(datafile,'w')
    dataf.write('{:>57}{:>3}{:>37}{:>38}{:>3}{:>38}'.format(sd,' Y+ ',sd+'-', \
        sd,' Y- ',sd+'-\n'))
    dataf.write(s)

    errf=open(errfile,'w')

    colormp=plt.cm.get_cmap('rainbow')
    pi2=2.0*math.pi

#------------ make plots on the tkinter window -----------------------------
    #clear figure if it exists
    if fig != None:
        fig.clf()
        
    #clear canvas if it exists
    if canv != None:
        canv.get_tk_widget().destroy()

    fig=Figure(figsize=(2.4,1.8),dpi=400)

    canv = figtk(fig, master=wd)
    canv.draw()
    canv.get_tk_widget().place(x=0,y=50)

    if toolbar != 0:
        toolbar.destroy()

    toolbar=navitk(canv, wd)
    toolbar.update()

    def on_key_press(event):
        key_press_handler(event, canv, toolbar)
        canv.mpl_connect("key_press_event", on_key_press)

    #set size, resolution, and position of axes
    xL=[0.08,0.08,0.08,0.52,0.52,0.52]
    yB=[0.70,0.44,0.13,0.68,0.42,0.16]
    wh=[0.36,0.36,0.36,0.35,0.35,0.35]
    ht=[0.20,0.20,0.20,0.21,0.21,0.21]

    sc=[0]*3
    axes=[]

    #fig=plt.figure(figsize=(1.9, 1.5), dpi=400)
    for i in range(6):
        axes.append(fig.add_axes([xL[i], yB[i], wh[i], ht[i]]))
        fig.add_axes(axes[i])

    #set line width of axes
    for ax in ['left','bottom','right','top']:
        for i in range(6):
            axes[i].spines[ax].set_linewidth(0.3)

    sL1='$L_1$ = '+str("{0:.4f}".format(data_all[0][2][1]))+' H'
    if bname == 'All-Bands':
        sL1='All-Bands'
        xscale='log'
    else:
        xscale='linear'

    abr=['Antenna Plus','Antenna Minus','Antenna Plus + Minus']

    nrecd=len(data_all)

    date=[]
    entv=[]
    entv3=[]
    Q=[]
    freq=[]
    Vap=[]
    phip=[]
    Vam=[]
    phim=[]
    inout=[]
    prog=[]
    #Va3p=[]
    #phi3p=[]
    #Va3m=[]
    #phi3m=[]

    fmins=[0]*nrecd
    fmaxs=[0]*nrecd

    for k in range(nrecd):
        date.append(data_all[k][0])
        entv.append(data_all[k][2])
        entv3.append(data_all[k][3])
        Q.append(data_all[k][4])
        freq.append(data_all[k][5])
        Vap.append(data_all[k][6])
        phip.append(data_all[k][8])
        Vam.append(data_all[k][10])
        phim.append(data_all[k][12])
        inout.append(data_all[k][14])
        prog.append(data_all[k][15])

        #Va3p.append(data_all[k][7])
        #phi3p.append(data_all[k][9])
        #Va3m.append(data_all[k][11])
        #phi3m.append(data_all[k][13])

        fmins[k]=min(data_all[k][5])
        fmaxs[k]=max(data_all[k][5])

    fmin=min(fmins)/1.0e3
    fmax=max(fmaxs)/1.0e3

    fres=[]
    Ca=np.zeros(nrecd)
    Ra=np.zeros(nrecd)
    pwr=[]
    Vaf=[]

    #diff=0
    diff1=0
    #diff2=0
    diff3=0
    diff4=0
    diff5=0
    diff6=0
    i=0

    for k in range(0,nrecd):
        if k == 0 or (k > 0 and date[k][0:11] != date[k-1][0:11]):
            print(date[k])

        #if (prog[k] != 54 and prog[k] != 59 and prog[k] != 64):
        #    diff=diff+1
        #    continue

        nfq=len(freq[k])

        fq=np.zeros(nfq)
        Va1=np.zeros(nfq)
        #Va31=np.zeros(nfq)
        phi1=np.zeros(nfq)
        #phi31=np.zeros(nfq)

        Va2=np.zeros(nfq)
        #Va32=np.zeros(nfq)
        phi2=np.zeros(nfq)
        #phi32=np.zeros(nfq)

        fq[0:nfq]=freq[k][0:nfq]

        Va1[0:nfq]=Vap[k][0:nfq] 
        Va2[0:nfq]=Vam[k][0:nfq] 

        #Va31[0:nfq]=Va3p[k][0:nfq]
        phi1[0:nfq]=phip[k][0:nfq]
        #phi31[0:nfq]=phi3p[k][0:nfq]

        #Va32[0:nfq]=Va3m[k][0:nfq]
        phi2[0:nfq]=phim[k][0:nfq]
        #phi32[0:nfq]=phi3m[k][0:nfq]

        nfsp=np.argmax(Va1)
        nfsm=np.argmax(Va2)

        #clculate Q using interpolated high resolution Va - freq data
        Qp=ant_impedance.calc_Q(Va1,fq)
        Qm=ant_impedance.calc_Q(Va2,fq)

        #clculate Q using original Va - freq data
        #Qpold=ant_impedance_v2.calc_Q(Va1,fq)
        #Qmold=ant_impedance_v2.calc_Q(Va2,fq)

        #log = entv[k][1] != entv[k][4]
        #log0 = entv[k][2] != entv[k][5]
        log1= Qp <= 0.0 or Qm <= 0.0
        #log2 =   (entv[k][1] == 0.0375 and fq[nfsp] >= 35.0e3) \
        #      or (entv[k][4] == 0.0375 and fq[nfsm] >= 35.0e3) \
        #      or (entv[k][1] == 0.3015 and fq[nfsp] >= 11.5e3) \
        #      or (entv[k][4] == 0.3015 and fq[nfsm] >= 11.5e3) \
        #      or (entv[k][1] == 2.1515 and fq[nfsp] >= 4.8e3)  \
        #      or (entv[k][4] == 2.1515 and fq[nfsm] >= 4.8e3)
        log3= entv3[k][2] > 0.0 or entv3[k][5] > 0.0

        log4=nfsp != nfsm

        #exclude data with following conditions
        #if log or log0 or log1 or log2 or log3:
        if log1 or log3 or log4:
            s1='{:4d}{:7.4f}{:6.1f}{:8.1f}{:6.1f}{:8.1f}{:8.1f}'  \
            .format(int(entv[k][0]),entv[k][1],entv[k][2],fq[nfsp], \
                entv3[k][2],Va1[nfsp],min(Va1))
            s2='{:4d}{:7.4f}{:6.1f}{:8.1f}{:6.1f}{:8.1f}{:8.1f}{:5d}' \
                .format(int(entv[k][3]),entv[k][4],entv[k][5],fq[nfsm], \
                entv3[k][5],Va2[nfsm],min(Va2),prog[k])

            #if log:
            #    errf.write('L1 of two TATUs are different:              ' \
            #        +('{:8.4f}'*2).format(entv[k][1],entv[k][4])+' '+date[k] \
            #         +'{:5d}'.format(prog[k])+'\n')
            #if log0:
            #    errf.write('C1 of two TATUs are different:              ' \
            #        +('{:7.1f}'*2).format(entv[k][2],entv[k][5])+' '+date[k] \
            #        +'{:5d}'.format(prog[k])+'\n')
            if log1:
                diff1=diff1+1
                errf.write('Possibly no resonance on one or both TATUs: ' \
                     +('{:10.2f}'*2).format(Qp,Qm)+' '+date[k]+'{:5d}' \
                    .format(prog[k])+'\n')
            #if log2: #not us log2
            #    diff2=diff2+1
                #errf.write('One or both reson frequencies beyond range: ' \
                #     +('{:10.2f}'*2).format(fq[nfsp],fq[nfsm])+' '+date[k] \
                #    +'{:5d}'.format(prog[k])+'\n')
            if log3:
                diff3=diff3+1
                errf.write('Third harmonic turned on one or both TATUs: ' \
                    +('{:10.2f}'*2).format(entv3[k][2],entv[k][5])+' '+date[k] \
                    +'{:5d}'.format(prog[k])+'\n')

            if log4:
                diff4=diff4+1
                errf.write('Resonance frequency of two TATUs are diffe: ' \
                    +('{:10.2f}'*2).format(entv3[k][2],entv[k][5])+' '+date[k] \
                    +'{:5d}'.format(prog[k])+'\n')

            logf.write(date[k]+s1+s2+'\n')

            continue

        QQ=[Qp,Qm]
        if symm == 'Yes':
            Rap, Ram, Cap, Cam, Iap, Iam = ant_impedance.Ca_Ra_symmetric(entv[k], entv3[k], \
                QQ, Va1, phi1, Va2, phi2, fq) #Va31, phi31, Va32, phi32, 
        else:
            Rap, Ram, Cap, Cam, Iap, Iam = ant_impedance.Ca_Ra_asymmetr(entv[k], entv3[k], \
                QQ, Va1, phi1, Va2, phi2, fq) #Va31, phi31, Va32, phi32, 

        Rthd=1.0e6
        log5=(Rap >= Rthd or Ram >= Rthd) or (Rap <= 0.0 or Ram <= 0.0)
        log6=Cap <= 0.0 or Cam <= 0.0
        if log5 or log6:
            if log5:
                diff5=diff5+1

            if log6:
                diff6=diff6+1

        if log5 or log6:
            s1='{:4d}{:7.4f}{:6.1f}{:8.1f}{:6.1f}{:8.1f}{:8.1f}'  \
                .format(int(entv[k][0]),entv[k][1],entv[k][2],fq[nfsp], \
                entv3[k][2],Va1[nfsp],min(Va1))
            s2='{:4d}{:7.4f}{:6.1f}{:8.1f}{:6.1f}{:8.1f}{:8.1f}{:5d}' \
                .format(int(entv[k][3]),entv[k][4],entv[k][5],fq[nfsm], \
                entv3[k][5],Va2[nfsm],min(Va2),prog[k])

            if log5:
                errf.write('Too large/minus Ra on one or both monopole: ' \
                    +('{:10.1f}'*2).format(Rap,Ram)+' '+date[k]+'{:5d}' \
                    .format(prog[k])+'\n')
            else:
                errf.write('Negative Ca values on one or both monopole: ' \
                    +('{:10.2f}'*2).format(Cap,Cam)+' '+date[k]+'{:5d}' \
                    .format(prog[k])+'\n')

            logf.write(date[k]+s1+s2+'\n')

            #QQ=[Qp,1.5*Qm]
            #Rap, Ram, Cap, Cam, Iap, Iam = ant_impedance.Ca_Ra(entv[k], entv3[k], QQ, \
            #    Va1, phi1, Va2, phi2, Va31, phi31, Va32, phi32, fq)
            #diff=diff+1
            continue

        fres.append(fq[nfsp]/1.0e3)

        Cs1p, R1p, Ltp=Cs_Rind_Lt.Cs_RL_Lt(entv[k][1], fq[nfsp])
        Cs1m, R1m, Ltm=Cs_Rind_Lt.Cs_RL_Lt(entv[k][4], fq[nfsm])

        #antena output power
        Pa1=0.5*Iap**2*Rap
        Pa2=0.5*Iam**2*Ram

        Ra[i]=Rap+Ram
        Ca[i]=Cap*Cam/(Cap+Cam)

        #radiation power (in W) at resonance frequency
        if symm=="Yes":
            pwr.append(Pa1+Pa2)
            Vaf.append(Va1[nfsp])
        else:
            pwr.append(Pa1)

        s1=('{:4d}{:7.4f}{:6.1f}'+'{:8.1f}'*2+'{:7.2f}{:8.2f}{:9.1f}{:8.1f}'  \
            +'{:8.1f}{:6.1f}').format(int(entv[k][0]),Ltp,entv[k][2], \
            fq[nfsp],Va1[nfsp],Qp,Iap*1.0e3,Rap,R1p,Cap,Cs1p*1.0e12)
        s2=('{:4d}{:7.4f}{:6.1f}'+'{:8.1f}'*2+'{:7.2f}{:8.2f}{:9.1f}{:8.1f}'  \
            +'{:8.1f}{:6.1f}{:4d}{:6d}'+'{:11.5f}'*2+'{:6.1f}'*2) \
            .format(int(entv[k][3]),Ltm, \
            entv[k][5],fq[nfsm],Va2[nfsm],Qm,Iam*1.0e3,Ram,R1m,Cam, \
            Cs1m*1.0e12,inout[k],prog[k],phi1[nfsp],phi2[nfsm],Pa1,Pa2)

        dataf.write(date[k]+s1+s2+'\n')

        for j in range(nfq):
            Va1[j]=Vap[k][j]/entv[k][0]  #normalize Vap by driving voltage
            Va2[j]=Vam[k][j]/entv[k][3]  #normalize Vam by driving voltage

        C1p=entv[k][2]*1.0e-12
        C1m=entv[k][5]*1.0e-12

        if C1p != 0.0:
            x=[np.sqrt(1.0/(entv[k][1]*C1p))/(pi2*1.0e3)]*nfq
            sc[0] = axes[0].scatter(x,fq/1.0e3,c=np.log10(Va1),cmap=colormp, \
                                    s=0.01, marker='o')
        if C1m != 0.0:
            x=[np.sqrt(1.0/(entv[k][4]*C1m))/(pi2*1.0e3)]*nfq
            sc[1] = axes[1].scatter(x,fq/1.0e3,c=np.log10(Va2),cmap=colormp, \
                                    s=0.01, marker='o')

        x=[fq[nfsp]/1.0e3]*nfq
        sc[2] = axes[2].scatter(x,fq/1.0e3,c=np.log10(Va1+Va2),cmap=colormp, \
                                s=0.01, marker='o')

        #if date[k][0:10] == '2021.05.31':
        #    print(date[k], Qp, Qpold)

        i=i+1

    logf.close()
    dataf.close()
    errf.close()

    """
    if diff > 0:
        if inpflag == 'In':
            message='data in plasmasphere are not used'
        elif inpflag == 'Out':
            message='data out of plasmasphere are not used'
        else:
            message='data are not used'
    """

    print(diff1+diff3+diff4+diff5,'data out of',nrecd, 'data are not used')
    print('Possibly no resonance on one or both TATUs: ',diff1)
    print('Third harmonic turned on one or both TATUs: ',diff3)
    print('Resonance frequency of two TATUs are diffe: ',diff4)
    print('Too large/minus Ra on one or both monopole: ',diff5)
    print('Negative Ca values on one or both monopole: ',diff6)

# set axis, labels, color bar etc for left three panels
    for j in range(3):
        axes[j].set_ylim(fmin,fmax)
        axes[j].tick_params(axis='both',which='major',length=1.3,width=0.4,pad=1, \
                        labelsize=2.4,left=True,right=True,bottom=True,top=True)
        axes[j].tick_params(axis='both',which='minor',length=0.8,width=0.4,pad=1, \
                        labelsize=0,left=True,right=True,bottom=True,top=True)
        axes[j].xaxis.set_minor_locator(AutoMinorLocator())
        axes[j].yaxis.set_minor_locator(AutoMinorLocator())
        if j == 0:
            axes[j].set_xticklabels([])

        axes[j].set_ylabel ('Frequency (kHz)', fontsize=2.5, labelpad=0.7)

        #color bar
        cbar=fig.colorbar(sc[j], ax=axes[j], shrink=0.9, pad=0.03, aspect=10)
        cbar.ax.tick_params(labelsize=1.9, length=0.85, width=0.4, pad=0.4)
        cbar.ax.set_ylabel('Normalized Voltage', fontsize=2.1, labelpad=1.1)
        cbar.outline.set_visible(False)
        axes[j].text(1.04,1.0,'Log Scale',ha='left',va='bottom',fontsize=1.9, \
                    transform=axes[j].transAxes)

        #add text
        if i < 2:
            axes[j].text(0,1.04,abr[i],ha='left',va='bottom',fontsize=2.2, \
                transform=axes[j].transAxes)
        else:
            axes[j].text(0,1.04,abr[j],ha='left',va='bottom',fontsize=2.2, \
                transform=axes[j].transAxes)

    axes[1].set_xlabel(r'$1/2\pi \sqrt{L_1 C_1}$ (kHz)',fontsize=2.5,labelpad=0.6)
    axes[2].set_xlabel(r'$1/2\pi \sqrt{L_1 C_{total}}$ (kHz)',fontsize=2.5, \
                       labelpad=0.6)
    fsmin=min(fres)
    fsmax=max(fres)
    axes[2].set_xlim(fsmin,fsmax)

    #----- plot antenna Ca ------
    # first bin the data
    nbin=int((fsmax-fmin)/bwd)
    stat=stats.binned_statistic(fres, Ca[0:i], 'sum', bins=nbin)
    bsum=stat[0]
    stat=stats.binned_statistic(fres, Ca[0:i], 'mean', bins=nbin)
    Cab=stat[0]
    bcnt=bsum/Cab

    fbin=np.zeros(nbin)
    fbin[0]=fmin+bwd/2.0
    for j in range(1,nbin):
        #fbin[j]=fbin[j-1]+bwd
        fbin[j]=0.5*(stat[1][j]+stat[1][j+1])

    axes[3].scatter(fres, Ca[0:i], s=0.01, marker='.',color='blue')
    sc=axes[3].scatter(fbin, Cab, c=bcnt, cmap=colormp, s=0.08, marker='s')
    axes[3].set_xlim(fsmin,fsmax)
    axes[3].set_xscale(xscale)
    ymin, ymax = axes[3].get_ylim()
    axes[3].set_ylim(ymin,ymax+20)
    axes[3].tick_params(axis='both',which='major',length=1.2,width=0.4,pad=1, \
            labelsize=2.2,left=True,right=True,bottom=True,top=True,direction='in')
    axes[3].tick_params(axis='both',which='minor',length=0.7,width=0.4,pad=1, \
            labelsize=0,left=True,right=True,bottom=True,top=True,direction='in')
    #axes[3].xaxis.set_minor_locator(AutoMinorLocator())
    axes[3].yaxis.set_minor_locator(AutoMinorLocator())
    axes[3].set_xticklabels([])
    axes[3].set_ylabel('$C_a$ (pF)',fontsize=2.5,labelpad=0.7)

    #add text - date and inductance
    datemax=max(date)
    datemin=min(date)
    if inpflag == 'In':
        txt=sL1+' / In Plasmasphere'
    elif inpflag == 'Out':
        txt=sL1+' / Out of Plasmasphere'
    else:
        txt=sL1+' / Inside and Outside Plasmasphere'

    axes[3].text(-0.15,1.3,datemin[0:10]+'$-$'+datemax[0:10],fontsize=2.5, \
                ha='center',va='bottom',transform=axes[3].transAxes)
    axes[3].text(-0.15,1.15,txt,fontsize=2.5,ha='center',va='bottom', \
                transform=axes[3].transAxes)

    # ---- vertical color bar ----
    #cbar=fig.colorbar(sc, ax=axes[3], shrink=0.9, pad=0.03, aspect=10)
    #cbar.ax.tick_params(labelsize=1.9, length=0.85, width=0.4, pad=0.4)
    #cbar.ax.set_ylabel('No. of Data Points', fontsize=2.1, labelpad=1.1)
    #cbar.outline.set_visible(False)

    # ------- binned Ra -------
    #Iaam=np.sqrt(pwr[0:i]/Ra[0:i])
    stat=stats.binned_statistic(fres, Ra[0:i]/1.0e3, 'mean', bins=nbin)
    Rab=stat[0]

    axes[4].scatter(fres, Ra[0:i]/1.0e3, s=0.01, marker='.',color='blue')
    sc=axes[4].scatter(fbin, Rab, c=bcnt, cmap=colormp, s=0.08, marker='s')
    axes[4].set_xlim(fsmin,fsmax)
    axes[4].set_xscale(xscale)
    axes[4].set_yscale('log')
    axes[4].tick_params(axis='both',which='major',length=1.2,width=0.4,pad=1, \
            labelsize=2.2,left=True,right=True,bottom=True,top=True,direction='in')
    axes[4].tick_params(axis='both',which='minor',length=0.7,width=0.4,pad=1, \
            labelsize=0,left=True,right=True,bottom=True,top=True,direction='in')
    axes[4].set_xticklabels([])
    axes[4].set_ylabel('$R_a$ ($10^{3}$ Ohm)',fontsize=2.5,labelpad=0.7)
    #axes[4].set_ylabel('$I_a$ (mA)',fontsize=2.2,labelpad=0.7)

    # ---- vertical color bar ----
    #cbar=fig.colorbar(sc, ax=axes[4], shrink=0.9, pad=0.03, aspect=10)
    #cbar.ax.tick_params(labelsize=1.9, length=0.85, width=0.4, pad=0.4)
    #cbar.ax.set_ylabel('No. of Data Points', fontsize=2.1, labelpad=1.1)
    #cbar.outline.set_visible(False)

    #-------- binned radiation power ------
    stat=stats.binned_statistic(fres, pwr[0:i], 'mean', bins=nbin)
    pwrb=stat[0]

    axes[5].scatter(fres, pwr[0:i], s=0.01, marker='.',color='blue')
    sc=axes[5].scatter(fbin, pwrb, c=bcnt, cmap=colormp, s=0.08, marker='s')
    axes[5].set_xlim(fsmin,fsmax)
    axes[5].set_xscale(xscale)
    axes[5].set_yscale('log')
    #ymin, ymax = axes[4].get_ylim()
    #axes[5].set_ylim(0,ymax+10)
    axes[5].tick_params(axis='both',which='major',length=1.2,width=0.4,pad=1, \
            labelsize=2.2,left=True,right=True,bottom=True,top=True,direction='in')
    axes[5].tick_params(axis='both',which='minor',length=0.7,width=0.4,pad=1, \
            labelsize=0,left=True,right=True,bottom=True,top=True,direction='in')
    axes[5].set_xlabel('Resonance Frequency (kHz)',fontsize=2.5,labelpad=0.7)
    axes[5].set_ylabel('Dissipated Power (W)',fontsize=2.5,labelpad=0.7)

    # ---- color bar ----
    axins = inset_axes(axes[5], width='70%', height='8%', loc='lower left', \
        bbox_to_anchor=(0.1,-0.46,1,1), bbox_transform=axes[5].transAxes)
    cbar=fig.colorbar(sc, cax=axins, shrink=0.9, pad=0.03, aspect=10, \
        fraction=0.1, orientation='horizontal',boundaries=None)
    cbar.ax.tick_params(labelsize=1.9, length=1.3, width=0.3, pad=0.4)
    cbar.ax.set_xlabel('No. of Data Points', fontsize=2.1, labelpad=1.1)
    cbar.outline.set_visible(False)

    print('Completed processing high power transmission data')

    return canv, fig, toolbar

#####################################################################################
#####################################################################################
def plot_VaRaCaPrad_comb(wd, canv, fig, toolbar, fpath, data_all, bwd, bname, inpflag):

    print('Processing high power transmission data ...')

    fdir=fpath.strip()
    inpflag=inpflag.strip()
    if fdir[len(fdir)-1] == '/':
        logfile=fdir+bname+'_'+inpflag+'_excluded_comb.dat'
        datafile=fdir+bname+'_'+inpflag+'_data_comb.dat'
        errfile=fdir+bname+'_'+inpflag+'_excluded_comb.log'
    else:
        logfile=fdir+'/'+bname+'_'+inpflag+'_excluded_comb.dat'
        datafile=fdir+'/'+bname+'_'+inpflag+'_data_comb.dat'
        errfile=fdir+'/'+bname+'_'+inpflag+'_excluded_comb.log'

    s=('{:>7}{:>10}'+'{:>6}'*3+'{:>8}{:>6}'+'{:>8}'*2+'{:>5}'+'{:>6}'*2 \
        +'{:>8}{:>6}'+'{:>8}'*2+'{:>8}').format('date','time','V1','L1', \
        'C1','fres','C3','maxVa','minVa','V1','L1','C1','fres','C3','maxVa', \
        'minVa','prog#\n')

    logf=open(logfile,'w')
    logf.write('{:>42}{:>48}'.format('Y+','Y-\n'))
    logf.write(s)

    sd='------------------------'
    s=(('{:^10}{:^10}{:^5}'+'{:^7}'*3+'{:>4}{:>7}'+'{:>7}'*2 \
        +'{:>5}{:>6}{:>7}{:>8}{:>6}{:>7}{:>7}'+'{:>7}'*2+'{:>9}{:>9}{:>8}{:>9}'
        +'{:>6}{:>9}{:>7}').format('date','time','V1','L1','C1','fres','Va+', \
        'Q+','RL','Cs','V1','L1','C1','fres','Va-','Q-','RL','Cs','fres','Ra', \
        'Ca','Va','Ia','Q','InOut','Prog#\n'))

    dataf=open(datafile,'w')
    dataf.write('{:>44}{:>3}{:>24}{:>25}{:>4}{:>24}'.format(sd,' Y+ ',sd, \
        sd,' Y- ',sd+'\n'))
    dataf.write(s)

    errf=open(errfile,'w')

    colormp=plt.cm.get_cmap('rainbow')
    pi2=2.0*math.pi

#------------ make plots on the tkinter window -----------------------------
    #clear figure if it exists
    if fig != None:
        fig.clf()
        
    #clear canvas if it exists
    if canv != None:
        canv.get_tk_widget().destroy()

    fig=Figure(figsize=(2.6,2.0),dpi=400)

    canv = figtk(fig, master=wd)
    canv.draw()
    canv.get_tk_widget().place(x=0,y=50)

    if toolbar != 0:
        toolbar.destroy()

    toolbar=navitk(canv, wd)
    toolbar.update()

    def on_key_press(event):
        key_press_handler(event, canv, toolbar)
        canv.mpl_connect("key_press_event", on_key_press)

    #set size, resolution, and position of axes
    xL=[0.08,0.08,0.08,0.49,0.49,0.49]
    yB=[0.68,0.46,0.24,0.68,0.46,0.24]
    wh=[0.38,0.38,0.38,0.38,0.38,0.38]
    ht=[0.20,0.20,0.20,0.20,0.20,0.20]

    sc=[0]*3
    axes=[]

    #fig=plt.figure(figsize=(1.9, 1.5), dpi=400)
    for i in range(6):
        axes.append(fig.add_axes([xL[i], yB[i], wh[i], ht[i]]))
        fig.add_axes(axes[i])

    #set line width of axes
    for ax in ['left','bottom','right','top']:
        for i in range(6):
            axes[i].spines[ax].set_linewidth(0.3)

    sL1='$L_1$ = '+str("{0:.4f}".format(data_all[0][2][1]))+' H'
    if bname == 'All-Bands':
        sL1='All-Bands'
        xscale='log'
    else:
        xscale='linear'

    abr=['Antenna Plus','Antenna Minus','Antenna Plus + Minus']

    nrecd=len(data_all)

    date=[]
    entv=[]
    entv3=[]
    Q=[]
    freq=[]
    Vap=[]
    phip=[]
    Vam=[]
    phim=[]
    inout=[]
    prog=[]
    Va3p=[]
    phi3p=[]
    Va3m=[]
    phi3m=[]

    fmins=[0]*nrecd
    fmaxs=[0]*nrecd

    for k in range(nrecd):
        date.append(data_all[k][0])
        entv.append(data_all[k][2])
        entv3.append(data_all[k][3])
        Q.append(data_all[k][4])
        freq.append(data_all[k][5])
        Vap.append(data_all[k][6])
        phip.append(data_all[k][8])
        Vam.append(data_all[k][10])
        phim.append(data_all[k][12])
        inout.append(data_all[k][14])
        prog.append(data_all[k][15])

        Va3p.append(data_all[k][7])
        phi3p.append(data_all[k][9])
        Va3m.append(data_all[k][11])
        phi3m.append(data_all[k][13])

        fmins[k]=min(data_all[k][5])
        fmaxs[k]=max(data_all[k][5])

    fmin=min(fmins)/1.0e3
    fmax=max(fmaxs)/1.0e3

    fres=[]
    Ca=np.zeros(nrecd)
    Ra=np.zeros(nrecd)
    pwr=[]

    diff=0
    i=0
    for k in range(0,nrecd):
        nfq=len(freq[k])
        Vat=np.zeros(nfq)

        fq=np.zeros(nfq)
        Va1=np.zeros(nfq)
        Va2=np.zeros(nfq)
        phi1=np.zeros(nfq)
        phi2=np.zeros(nfq)

        Va31=np.zeros(nfq)
        Va32=np.zeros(nfq)
        phi31=np.zeros(nfq)
        phi32=np.zeros(nfq)

        fq[0:nfq]=freq[k][0:nfq]

        Va1[0:nfq]=Vap[k][0:nfq]  #wiil be normalized by driving voltage
        Va2[0:nfq]=Vam[k][0:nfq]  #will be normalized by driving voltage
        phi1[0:nfq]=phip[k][0:nfq]
        phi2[0:nfq]=phim[k][0:nfq]

        Va31[0:nfq]=Va3p[k][0:nfq]
        Va32[0:nfq]=Va3m[k][0:nfq]
        phi31[0:nfq]=phi3p[k][0:nfq]
        phi32[0:nfq]=phi3m[k][0:nfq]

        nfsp=np.argmax(Va1)
        nfsm=np.argmax(Va2)

        log0 = entv[k][1] != entv[k][4]
        log1 = entv[k][2] != entv[k][5]
        log2 =   (entv[k][1] == 0.0375 and fq[nfsp] >= 35.0e3) \
              or (entv[k][4] == 0.0375 and fq[nfsm] >= 35.0e3) \
              or (entv[k][1] == 0.3015 and fq[nfsp] >= 11.5e3) \
              or (entv[k][4] == 0.3015 and fq[nfsm] >= 11.5e3) \
              or (entv[k][1] == 2.1515 and fq[nfsp] >= 4.8e3)  \
              or (entv[k][4] == 2.1515 and fq[nfsm] >= 4.8e3)
        log3= entv3[k][2] > 0.0 or entv3[k][5] > 0.0

        #exclude data with following conditions
        #if log2 or log3:
        if log0 or log1 or log3:
            s1='{:4d}{:7.4f}{:6.1f}{:8.1f}{:6.1f}{:8.1f}{:8.1f}'  \
                .format(int(entv[k][0]),entv[k][1],entv[k][2],fq[nfsp], \
                entv3[k][2],Va1[nfsp],min(Va1))
            s2='{:4d}{:7.4f}{:6.1f}{:8.1f}{:6.1f}{:8.1f}{:8.1f}{:5d}' \
                .format(int(entv[k][3]),entv[k][4],entv[k][5],fq[nfsm], \
                entv3[k][5],Va2[nfsm],min(Va2),prog[k])

            if log0:
                #print('L1 of two TATUs are different: ',('{:8.4f}'*2)
                #    .format(entv[k][1],entv[k][4]),' '+date[k]+'{:5d}'.format(prog[k]))
                errf.write('C1 of two TATUs are different: ' \
                    +('{:8.4f}'*2).format(entv[k][1],entv[k][4])+' '+date[k] \
                    +'{:5d}'.format(prog[k])+'\n')
            if log1:
                #print('C1 of two TATUs are different: ',('{:7.1f}'*2)
                #    .format(entv[k][2],entv[k][5]),' '+date[k]+'{:5d}'.format(prog[k]))
                errf.write('C1 of two TATUs are different: ' \
                    +('{:7.1f}'*2).format(entv[k][2],entv[k][5])+' '+date[k] \
                    +'{:5d}'.format(prog[k])+'\n')
            if log2:
                #print('One or both reson frequencies beyond range: ',('{:9.2f}'*2)
                #    .format(fq[nfsp],fq[nfsm]),' '+date[k]+'{:5d}'.format(prog[k]))
                errf.write('One or both reson frequencies beyond range: ' \
                    +('{:10.2f}'*2).format(fq[nfsp],fq[nfsm])+' '+date[k] \
                    +'{:5d}'.format(prog[k])+'\n')
            elif log3:
                #print('Third harmonic turned on one or both TATUs: ',('{:9.2f}'*2) \
                #    .format(entv3[k][2],entv[k][5]),' '+date[k]+'{:5d}'.format(prog[k]))
                errf.write('Third harmonic turned on one or both TATUs: ' \
                    +('{:10.2f}'*2).format(entv3[k][2],entv[k][5])+' '+date[k] \
                    +'{:5d}'.format(prog[k])+'\n')

            logf.write(date[k]+s1+s2+'\n')

            diff=diff+1
            continue

        Ra[i], Ca[i], Vat, Ia, QQ = ant_impedance.Ca_Ra_comb(entv[k], entv3[k], \
                Va1, phi1, Va2, phi2, Va31, phi31, Va32, phi32, fq)

        if QQ <= 0.0:
            #print('Possibly no resonance on one or both TATUs: ',('{:9.2f}'*2) \
            #    .format(Qp,Qm),' '+date[k]+'{:5d}'.format(prog[k]))
            errf.write('Possibly no resonance on one or both TATUs: ' \
                +('{:10.2f}'*2).format(Qp,Qm)+' '+date[k]+'{:5d}'.format(prog[k])+'\n')

            s1='{:4d}{:7.4f}{:6.1f}{:8.1f}{:6.1f}{:8.1f}{:8.1f}'  \
                .format(int(entv[k][0]),entv[k][1],entv[k][2],fq[nfsp], \
                entv3[k][2],Va1[nfsp],min(Va1))
            s2='{:4d}{:7.4f}{:6.1f}{:8.1f}{:6.1f}{:8.1f}{:8.1f}{:5d}' \
                .format(int(entv[k][3]),entv[k][4],entv[k][5],fq[nfsm], \
                entv3[k][5],Va2[nfsm],min(Va2),prog[k])

            logf.write(date[k]+s1+s2+'\n')

            diff=diff+1
            continue

        Rthd=1.0e6
        if Rthd >= 1.0e6 or Ra[i] <= 0.0:
            s1='{:4d}{:7.4f}{:6.1f}{:8.1f}{:6.1f}{:8.1f}{:8.1f}'  \
                .format(int(entv[k][0]),entv[k][1],entv[k][2],fq[nfsp], \
                entv3[k][2],Va1[nfsp],min(Va1))
            s2='{:4d}{:7.4f}{:6.1f}{:8.1f}{:6.1f}{:8.1f}{:8.1f}{:5d}' \
                .format(int(entv[k][3]),entv[k][4],entv[k][5],fq[nfsm], \
                entv3[k][5],Va2[nfsm],min(Va2),prog[k])
            #print('Too large or negative antenna resistance:', \
            #    '{:10.2f}'.format(Ra[i]),' '+date[k]+'{:5d}'.format(prog[k]))
            errf.write('Too large or negative antenna resistance:' \
                +'{:11.1f}'.format(Ra[i])+' '+date[k]+'{:5d}'.format(prog[k])+'\n')

            logf.write(date[k]+s1+s2+'\n')

            diff=diff+1
            continue

        if Ca[i] <= 0.0:
            s1='{:4d}{:7.4f}{:6.1f}{:8.1f}{:6.1f}{:8.1f}{:8.1f}'  \
                .format(int(entv[k][0]),entv[k][1],entv[k][2],fq[nfsp], \
                entv3[k][2],Va1[nfsp],min(Va1))
            s2='{:4d}{:7.4f}{:6.1f}{:8.1f}{:6.1f}{:8.1f}{:8.1f}{:5d}' \
                .format(int(entv[k][3]),entv[k][4],entv[k][5],fq[nfsm], \
                entv3[k][5],Va2[nfsm],min(Va2),prog[k])
            #print('Unrealistic Negative antenna capacitance:', \
            #    '{:10.2f}'.format(Ca[i]),' '+date[k]+'{:5d}'.format(prog[k]))
            errf.write('Unrealistic Negative antenna capacitance:' \
                +'{:10.2f}'.format(Ca[i])+' '+date[k]+'{:5d}'.format(prog[k])+'\n')

            logf.write(date[k]+s1+s2+'\n')

            diff=diff+1
            continue

        nfs=np.argmax(Vat)
        fres.append(fq[nfs]/1.0e3)

        #radiation power (in W) at resonance frequency
        pwr.append(0.5*Ia**2*Ra[i])

        Cs1p, R1p, Ltp=Cs_Rind_Lt.Cs_RL_Lt(entv[k][1], fq[nfsp])
        Cs1m, R1m, Ltm=Cs_Rind_Lt.Cs_RL_Lt(entv[k][4], fq[nfsm])
        Qp=Q[k][0]
        Qm=Q[k][1]

        s1=('{:4d}{:7.4f}{:6.1f}'+'{:8.1f}'*2+'{:7.2f}{:7.1f}'  \
            +'{:6.1f}').format(int(entv[k][0]),Ltp,entv[k][2], \
            fq[nfsp],Va1[nfsp],Qp,R1p,Cs1p*1.0e12)
        s2=('{:4d}{:7.4f}{:6.1f}'+'{:8.1f}'*2+'{:7.2f}{:7.1f}{:6.1f}'  \
            +'{:8.1f}{:10.2f}{:9.2f}{:8.1f}{:8.2f}{:7.2f}{:4d}{:6d}') \
            .format(int(entv[k][3]),Ltm,entv[k][5],fq[nfsm],Va2[nfsm], \
            Qm,R1m,Cs1m*1.0e12,fq[nfs],Ra[i],Ca[i],Vat[nfs],Ia*1.0e3,QQ, \
            inout[k],prog[k])

        dataf.write(date[k]+s1+s2+'\n')

        for j in range(nfq):
            Va1[j]=Vap[k][j]/entv[k][0]  #normalize Vap by driving voltage
            Va2[j]=Vam[k][j]/entv[k][3]  #normalize Vam by driving voltage
            Vat[j]=Vat[j]/(entv[k][0]+entv[k][3])

        C1p=entv[k][2]*1.0e-12
        C1m=entv[k][5]*1.0e-12

        if C1p != 0.0:
            x=[np.sqrt(1.0/(entv[k][1]*C1p))/(pi2*1.0e3)]*nfq
            sc[0] = axes[0].scatter(x,fq/1.0e3,c=np.log10(Va1),cmap=colormp, \
                                    s=0.01, marker='o')
        if C1m != 0.0:
            x=[np.sqrt(1.0/(entv[k][4]*C1m))/(pi2*1.0e3)]*nfq
            sc[1] = axes[1].scatter(x,fq/1.0e3,c=np.log10(Va2),cmap=colormp, \
                                    s=0.01, marker='o')

        x=[fq[nfs]/1.0e3]*nfq
        sc[2] = axes[2].scatter(x,fq/1.0e3,c=np.log10(Vat),cmap=colormp, \
                                s=0.01, marker='o')

        i=i+1

    logf.close()
    dataf.close()
    errf.close()

    if diff > 0:
        if inpflag == 'In':
            message='data outside plasmasphere are not used'
        elif inpflag == 'Out':
            message='data inside of plasmasphere are not used'
        else:
            message='data are not used'

        print(diff,'data out of',nrecd, message)

# set axis, labels, color bar etc for left three panels
    for j in range(3):
        axes[j].set_ylim(fmin,fmax)
        axes[j].tick_params(axis='both',which='major',length=1.3,width=0.4,pad=1, \
                        labelsize=2.4,left=True,right=True,bottom=True,top=True)
        axes[j].tick_params(axis='both',which='minor',length=0.8,width=0.4,pad=1, \
                        labelsize=0,left=True,right=True,bottom=True,top=True)
        axes[j].xaxis.set_minor_locator(AutoMinorLocator())
        axes[j].yaxis.set_minor_locator(AutoMinorLocator())
        if j == 0:
            axes[j].set_xticklabels([])

        axes[j].set_ylabel ('Frequency (kHz)', fontsize=2.5, labelpad=0.7)

        #color bar
        cbar=fig.colorbar(sc[j], ax=axes[j], shrink=0.9, pad=0.03, aspect=10)
        cbar.ax.tick_params(labelsize=1.9, length=0.85, width=0.4, pad=0.4)
        cbar.ax.set_ylabel('Normalized Voltage', fontsize=2.1, labelpad=1.1)
        cbar.outline.set_visible(False)
        axes[j].text(1.04,1.0,'Log Scale',ha='left',va='bottom',fontsize=1.9, \
                    transform=axes[j].transAxes)

        #add text
        if j < 2:
            axes[j].text(0,1.04,abr[j],ha='left',va='bottom',fontsize=2.2, \
                transform=axes[j].transAxes)
        else:
            axes[j].text(0,1.04,abr[j],ha='left',va='bottom',fontsize=2.2, \
                transform=axes[j].transAxes)

    axes[1].set_xlabel(r'$1/2\pi \sqrt{L_1 C_1}$ (kHz)',fontsize=2.5,labelpad=0.6)
    axes[2].set_xlabel(r'$1/2\pi \sqrt{L_1 C_{total}}$ (kHz)',fontsize=2.5, \
                       labelpad=0.6)
    fsmin=min(fres)
    fsmax=max(fres)
    axes[2].set_xlim(fsmin,fsmax)

    #----- plot antenna Ca ------
    # first bin the data
    nbin=int((fsmax-fmin)/bwd)
    stat=stats.binned_statistic(fres, Ca[0:i], 'sum', bins=nbin)
    bsum=stat[0]
    stat=stats.binned_statistic(fres, Ca[0:i], 'mean', bins=nbin)
    Cab=stat[0]
    bcnt=bsum/Cab

    fbin=np.zeros(nbin)
    fbin[0]=fmin+bwd/2.0
    for j in range(1,nbin):
        #fbin[j]=fbin[j-1]+bwd
        fbin[j]=0.5*(stat[1][j]+stat[1][j+1])

    axes[3].scatter(fres, Ca[0:i], s=0.01, marker='.',color='blue')
    sc=axes[3].scatter(fbin, Cab, c=bcnt, cmap=colormp, s=0.08, marker='s')
    axes[3].set_xlim(fsmin,fsmax)
    axes[3].set_xscale(xscale)
    ymin, ymax = axes[3].get_ylim()
    axes[3].set_ylim(ymin,ymax+20)
    axes[3].tick_params(axis='both',which='major',length=1.2,width=0.4,pad=1, \
            labelsize=2.2,left=True,right=True,bottom=True,top=True,direction='in')
    axes[3].tick_params(axis='both',which='minor',length=0.7,width=0.4,pad=1, \
            labelsize=0,left=True,right=True,bottom=True,top=True,direction='in')
    #axes[3].xaxis.set_minor_locator(AutoMinorLocator())
    axes[3].yaxis.set_minor_locator(AutoMinorLocator())
    axes[3].set_xticklabels([])
    axes[3].set_ylabel('$C_a$ (pF)',fontsize=2.5,labelpad=0.7)

    #add text - date and inductance
    datemax=max(date)
    datemin=min(date)
    if inpflag == 'In':
        txt=sL1+' / In Plasmasphere'
    elif inpflag == 'Out':
        txt=sL1+' / Out of Plasmasphere'
    else:
        txt=sL1+' / Inside and Outside Plasmasphere'

    axes[3].text(-0.15,1.3,datemin[0:10]+'$-$'+datemax[0:10],fontsize=2.5, \
                ha='center',va='bottom',transform=axes[3].transAxes)
    axes[3].text(-0.15,1.15,txt,fontsize=2.5,ha='center',va='bottom', \
                transform=axes[3].transAxes)

    # ---- vertical color bar ----
    #cbar=fig.colorbar(sc, ax=axes[3], shrink=0.9, pad=0.03, aspect=10)
    #cbar.ax.tick_params(labelsize=1.9, length=0.85, width=0.4, pad=0.4)
    #cbar.ax.set_ylabel('No. of Data Points', fontsize=2.1, labelpad=1.1)
    #cbar.outline.set_visible(False)

    # ------- binned Ra -------
    #Iaam=np.sqrt(pwr[0:i]/Ra[0:i])
    stat=stats.binned_statistic(fres, Ra[0:i]/1.0e3, 'mean', bins=nbin)
    Rab=stat[0]

    axes[4].scatter(fres, Ra[0:i]/1.0e3, s=0.01, marker='.',color='blue')
    sc=axes[4].scatter(fbin, Rab, c=bcnt, cmap=colormp, s=0.08, marker='s')
    axes[4].set_xlim(fsmin,fsmax)
    axes[4].set_xscale(xscale)
    axes[4].set_yscale('log')
    axes[4].tick_params(axis='both',which='major',length=1.2,width=0.4,pad=1, \
            labelsize=2.2,left=True,right=True,bottom=True,top=True,direction='in')
    axes[4].tick_params(axis='both',which='minor',length=0.7,width=0.4,pad=1, \
            labelsize=0,left=True,right=True,bottom=True,top=True,direction='in')
    axes[4].set_xticklabels([])
    axes[4].set_ylabel('$R_a$ ($10^{3}$ Ohm)',fontsize=2.5,labelpad=0.7)
    #axes[4].set_ylabel('$I_a$ (mA)',fontsize=2.2,labelpad=0.7)

    # ---- vertical color bar ----
    #cbar=fig.colorbar(sc, ax=axes[4], shrink=0.9, pad=0.03, aspect=10)
    #cbar.ax.tick_params(labelsize=1.9, length=0.85, width=0.4, pad=0.4)
    #cbar.ax.set_ylabel('No. of Data Points', fontsize=2.1, labelpad=1.1)
    #cbar.outline.set_visible(False)

    #-------- binned radiation power ------
    stat=stats.binned_statistic(fres, pwr[0:i], 'mean', bins=nbin)
    pwrb=stat[0]

    axes[5].scatter(fres, pwr[0:i], s=0.01, marker='.',color='blue')
    sc=axes[5].scatter(fbin, pwrb, c=bcnt, cmap=colormp, s=0.08, marker='s')
    axes[5].set_xlim(fsmin,fsmax)
    axes[5].set_xscale(xscale)
    axes[5].set_yscale('log')
    #ymin, ymax = axes[4].get_ylim()
    #axes[5].set_ylim(0,ymax+10)
    axes[5].tick_params(axis='both',which='major',length=1.2,width=0.4,pad=1, \
            labelsize=2.2,left=True,right=True,bottom=True,top=True,direction='in')
    axes[5].tick_params(axis='both',which='minor',length=0.7,width=0.4,pad=1, \
            labelsize=0,left=True,right=True,bottom=True,top=True,direction='in')
    axes[5].set_xlabel('Resonance Frequency (kHz)',fontsize=2.5,labelpad=0.7)
    axes[5].set_ylabel('Dissipated Power (W)',fontsize=2.5,labelpad=0.7)

    # ---- color bar ----
    axins = inset_axes(axes[5], width='70%', height='8%', loc='lower left', \
        bbox_to_anchor=(0.1,-0.46,1,1), bbox_transform=axes[5].transAxes)
    cbar=fig.colorbar(sc, cax=axins, shrink=0.9, pad=0.03, aspect=10, \
        fraction=0.1, orientation='horizontal',boundaries=None)
    cbar.ax.tick_params(labelsize=1.9, length=1.3, width=0.3, pad=0.4)
    cbar.ax.set_xlabel('No. of Data Points', fontsize=2.1, labelpad=1.1)
    cbar.outline.set_visible(False)

    print('Completed processing high power transmission data')

    return canv, fig, toolbar

#####################################################################################
#####################################################################################
#-------------------- plot Ra, Ca, and Prad only for Y + and Y- ---------------------------
def plot_Ra_Ca_Prad(wd, canv, fig, toolbar, fpath, bwd, L, bname, inpflag, \
    V0, comb, corx):

    print('Read calculated antenna parameters ...')
    fdir=fpath.strip()
    inpflag=inpflag.strip()
    if fdir[len(fdir)-1] == '/':
        datafile=fdir+'All-Bands_InOut_data-Nature.dat'
    else:
        datafile=fdir+'/All-Bands_InOut_data-Nature.dat'

    with open(datafile,'r') as dataf:
        dataf.readline()
        dataf.readline()

        fl=dataf.readlines()

        dataf.close()

    print('Finished reading data')

    nrecd=len(fl)

    colormp=plt.cm.get_cmap('rainbow')

#------------ make plots on the tkinter window -----------------------------
    #clear figure if it exists
    if fig != None:
        fig.clf()
        
    #clear canvas if it exists
    if canv != None:
        canv.get_tk_widget().destroy()

    fig=Figure(figsize=(2.8,2.3),dpi=400)
    #fig=Figure(figsize=(1.67,1.95),dpi=400)

    canv = figtk(fig, master=wd)
    canv.draw()
    canv.get_tk_widget().place(x=0,y=48)

    if toolbar != 0:
        toolbar.destroy()

    toolbar=navitk(canv, wd)
    toolbar.update()

    def on_key_press(event):
        key_press_handler(event, canv, toolbar)
        canv.mpl_connect("key_press_event", on_key_press)

    #set size, resolution, and position of axes
    xL=[0.08,0.08,0.08,0.49,0.49,0.49]
    yB=[0.69,0.475,0.44,0.26,0.48,0.26]
    wh=[0.40,0.40,0.40,0.38,0.38,0.38]
    ht=[0.17,0.17,0.17,0.17,0.17,0.17]

    if comb == 'Yes':
        ndata=1
        nplt=4
    else:
        ndata=1
        nplt=4

    sc=[0]*nplt
    axes=[]

    for i in range(nplt):
        axes.append(fig.add_axes([xL[i], yB[i], wh[i], ht[i]]))
        fig.add_axes(axes[i])

    #set line width of axes
    for ax in ['left','bottom','right','top']:
        for i in range(nplt):
            axes[i].spines[ax].set_linewidth(0.3)

    sL1='$L_1$ = '+str("{0:.4f}".format(L))+' H'
    if bname == 'All-Bands':
        sL1=''
    if V0 == 9696:
        sL1=sL1+' V1 >= 96 V'
    elif V0 == 8080:
        sL1=sL1+' V1 >= 80 V'
    elif V0 == 7272:
        sL1=sL1+' V1 < 72 V'

    if bname == 'All-Bands':
        if comb == 'Yes':
            if corx == 'Ca':
                minCX=200.0
                maxCX=450.0
            else:
                minCX=15e3
                maxCX=400e3
            minRa=110 
            maxRa=7e4
            minPr=0.1
            maxPr=60.0
            if V0 == 8080:
                minRa=110
                maxRa=7e4
                minPr=0.5
                maxPr=60.0
        else:
            if corx == 'Ca':
                minCX=400.0
                maxCX=900.0
            else:
                minCX=17e3
                maxCX=400e3
            minRa=20
            maxRa=65e3
            minPr=0.1
            maxPr=25.0
            if V0 == 8080:
                minRa=0.1e3
                maxRa=100e3
                minPr=0.2
                maxPr=60.0
    elif bname == 'High-Band':
        if corx == 'Ca':
            minCX=400.0
            maxCX=700.0
        else:
            minCX=18
            maxCX=27
        minRa=0.2
        maxRa=3.0
        minPr=10
        maxPr=100.0
    elif bname == 'Mid-Band':
        if corx == 'Ca':
            minCX=450.0
            maxCX=850.0
        else:
            minCX=18
            maxCX=27
        minRa=0.2
        maxRa=10.0
        if corx == 'Ca':
            minCX=450.0
            maxCX=900.0
        else:
            minCX=18
            maxCX=27
        minRa=0.3
        maxRa=40.0
        minPr=0.2
        maxPr=6.0

    pi=math.pi
    pi2=2.0*pi
    LL=[0.0375, 0.3015, 2.1515]

    date=[]
    fres=[[],[]]
    CaXa=[[],[]]
    Ra=[[],[]]
    pw=[[],[]]
    Vapm=[[],[]]

    diff=0
    for k in range(0,nrecd):
        a=fl[k].split()
        datet=a[0]
        L1=float(a[3])
        inpl=int(a[24])

        if V0 == 9696 and float(a[2]) < 96.0:
            print('Driving voltage is < ',96)
            diff=diff+1
            continue
        elif V0 == 8080 and float(a[2]) < 80.0:
            print('Driving voltage is < ',80)
            diff=diff+1
            continue
        elif V0 == 7272 and float(a[2]) > 72.00:
            print('Driving voltage is > ',72)
            diff=diff+1
            continue

        if bname == 'High-Band':
            if L1 > 8.0*LL[0]:
                print('Exclude other bands:',('{:7.4f}'*2).format(L1,L)+ \
                    ' '+datet+' '+a[1])
                diff=diff+1
                continue
        elif bname == 'Mid-Band':
            if L1 < 0.02*LL[1] or L1 > 5.0*LL[0]:
                print('Exclude other bands:',('{:7.4f}'*2).format(L1,L)+ \
                    ' '+datet+' '+a[1])
                diff=diff+1
                continue
        elif bname == 'Low-Band':
            if L1 < 0.5*LL[2]:
                print('Exclude other bands:',('{:7.4f}'*2).format(L1,L)+ \
                    ' '+datet+' '+a[1])
                diff=diff+1
                continue

        if inpflag != 'InOut':
            if inpflag == 'In': #skip if not in plasmasphere
                if inpl == 0 or inpl == 2 or inpl == 3:
                    print('Out of the plasmasphere:','{:2d}'.format(inpl)+ \
                        ' '+datet+' '+a[1])
                    diff=diff+1
                    continue
            else:
                if inpl == 1:  #skip if in the plasmasphere
                    print('In the plasmasphere:','{:2d}'.format(inpl)+ \
                        ' '+datet+' '+a[1])
                    diff=diff+1
                    continue

        fqp=float(a[5])
        fqm=float(a[16])
        Ra1=float(a[9])                        #Ra in Ohm
        Ra2=float(a[20])

        if(L1 < 0.06 and (fqp>32e3 or fqm>32e3)):
            diff=diff+1
            continue
        if ((L1 > 0.1 and L1 < 0.5) and (fqp>10.3e3 or fqm>10.3e3)):
            diff=diff+1
            continue
        if (L1 > 1.0 and (fqp > 4e3 or fqm >4e3)):
            diff=diff+1
            continue

        Ca1=float(a[11])            # antenna Ca in pF
        Ca2=float(a[22])
        #Ia1=float(a[8])*1.0e-3      # Ia in A
        #Ia2=float(a[19])*1.0e-3

        V1p=float(a[2])
        V1m=float(a[13])
        RLp=float(a[10])
        RLm=float(a[21])

        Rtotp=2.0*math.pi*fqp*float(a[3])/float(a[7])
        Itotp=V1p/Rtotp
        Rtotm=2.0*math.pi*fqm*float(a[14])/float(a[18])
        Itotm=V1m/Rtotm

        prad1=0.5*Itotp**2*(Rtotp-RLp)        # Power in W
        prad2=0.5*Itotm**2*(Rtotm-RLm)

        if comb=='Yes':
            Ra1=Ra1+Ra2
            Ca1=Ca1*Ca2/(Ca1+Ca2)
            prad1=prad1+prad2

        if corx == 'Xa':
            # measured antenna reactance +/- branches in Ohm
            Xa1=1.0e12/(pi2*fqp*Ca1)
            Xa2=1.0e12/(pi2*fqm*Ca2)

            CaXa[0].append(np.log10(Xa1))
            CaXa[1].append(np.log10(Xa2))
        else:
            CaXa[0].append(Ca1)
            CaXa[1].append(Ca2)

        date.append(datet)

        fres[0].append(fqp/1.0e3)          #frequency in kHz
        Ra[0].append(np.log10(Ra1))  #log scale in Ohm
        pw[0].append(np.log10(prad1))      #logscale in W
        Vapm[0].append(np.log10(float(a[6])))

        fres[1].append(fqm/1.0e3)
        Ra[1].append(np.log10(Ra2))
        pw[1].append(np.log10(prad2))
        Vapm[1].append(np.log10(float(a[17])))
    #end 'for loop' for calculating antenna parameters

    if diff > 0:
        print(diff,'data out of',nrecd,'not used in analysis')

    # -------- calculate mean values in frequency bins ---------------
    #Cab, Rab, pwb, bcnt, fbin = binned_values.mean_values(bname,bwd,inpflag, \
    #        fres,fsmin,fsmax,Ca,Ra,pwr)

    fsmin=[min(fres[0]), min(fres[1])]
    fsmax=[max(fres[0]), max(fres[1])]

    if bname == 'All-Bands':
        xscale='log'
        if corx == 'Ca':
            yscale='linear'
        else:
            yscale='log'
        fmin=2.5
        fmax=40
    elif bname == 'High-Band':
        xscale='linear'
        yscale='linear'
        fmin=23
        fmax=32
    else:
        xscale='linear'
        yscale='linear'
        fmin=min([min(fres[0]), min(fres[1])])
        fmax=min([max(fres[0]), max(fres[1])])

    # --- calculate dipole antenna capacitance and resistance in vacuum
    xbin=int(fmax+2-fmin)
    freq=[0]*xbin
    Rvac=[0]*xbin
    Cvac=[0]*xbin

    # theoretical values of capacitance & resistance of DSX 3 wire dipole in vacuum
    La=41.0
    raa=0.2e-3
    for j in range(xbin):
        freq[j]=fmin+float(j)

        wlen=3.0e8/(freq[j]*1.0e3)
        Omega=pi2*freq[j]*1.0e3
        Rvac[j]=20.0*(pi*La/wlen)**2      # vacuum dipole resistance 1-wire
        Xvac=120.0*(np.log(La/(2.0*raa))-1.0)/np.tan(pi*La/wlen)
        Cvac[j]=1.0e12/(Omega*Xvac)       # vacuum dipole Capacitance 1- wire
        if comb == 'Yes':
            #double vacuum dipole Ca for one element & 3 wires
            Cvac[j]=Cvac[j]*3.0
            Rvac[j]=Rvac[j]/3.0
        else:
            Cvac[j]=Cvac[j]*3.0
            Rvac[j]=0.5*Rvac[j]/3.0

    print('Min & max vacuum Ca:',("{:8.2f}"*2).format(min(Cvac),max(Cvac)))
    N=8   # running average window width

    #-- for dot by dot plots of data points in each bin: y-axis
    ybin=80 # for 2D binning not for 1D
    y=np.zeros((3,ybin+1))
    yCX=np.zeros(ybin)
    yRa=np.zeros(ybin)
    ypw=np.zeros(ybin)

    sc=[0]*3
    for i in range(ndata):
        if i == 0:
            kb=0
        else:
            kb=3

        xbin=int((fsmax[i]-fsmin[i])/bwd)
        x=[0]*(xbin+1)
        xfr=[0]*xbin
        for k in range(xbin+1):
            x[k]=fsmin[i]+float(k)*bwd
            if k > 0:
                xfr[k-1]=0.5*(x[k-1]+x[k])

        #--- determine y min & max
        CXmin=min(CaXa[i])
        CXmax=max(CaXa[i])
        Ramin=min(Ra[i])
        Ramax=max(Ra[i])
        pwmin=min(pw[i])
        pwmax=max(pw[i])

        pwmin=min(Vapm[i])
        pwmax=max(Vapm[i])

        dCX=(CXmax-CXmin)/ybin
        dRa=(Ramax-Ramin)/ybin
        dpw=(pwmax-pwmin)/ybin

        #--- bin size and center of bins for Ca, Ra and pwr
        for k in range(ybin+1):
            y[0][k]=CXmin+float(k)*dCX
            y[1][k]=Ramin+float(k)*dRa
            y[2][k]=pwmin+float(k)*dpw
            if k > 0:
                yCX[k-1]=0.5*(y[0][k-1]+y[0][k])
                if corx == 'Xa':
                    yCX[k-1]=10.0**yCX[k-1]

                yRa[k-1]=0.5*(y[1][k-1]+y[1][k])
                yRa[k-1]=10.0**yRa[k-1]

                ypw[k-1]=0.5*(y[2][k-1]+y[2][k])
                ypw[k-1]=10.0**ypw[k-1]

        # --- calculate number of Ca data points in bins [x, y]
        counts=stats.binned_statistic_2d(fres[i],CaXa[i],fres[i],statistic='count', \
            bins=[x,y[0]])
        a=counts.statistic  # number of data points for Ca
        a=a.transpose()

        rdl=len(a[0])+1-N
        CXb2d=np.zeros((len(a), rdl))
        for k in range(len(a)):
            CXb2d[k]=moving_average.data_points_running_average(a[k], N)
        CXb2d=CXb2d.transpose()

        # --- calculate number of Ra data points in bins [x, y]
        counts=stats.binned_statistic_2d(fres[i],Ra[i],fres[i],statistic='count', \
            bins=[x,y[1]])
        a=counts.statistic  # number of data points for Ra
        a=a.transpose()

        rdl=len(a[0])+1-N
        Rab2d=np.zeros((len(a), rdl))
        for k in range(len(a)):
            Rab2d[k]=moving_average.data_points_running_average(a[k], N)
        Rab2d=Rab2d.transpose()

        # --- calculate number of power points in bins [x, y]
        counts=stats.binned_statistic_2d(fres[i],Vapm[i],fres[i],statistic='count', \
            bins=[x,y[2]])
        a=counts.statistic  # number of data points for pwr
        a=a.transpose()

        rdl=len(a[0])+1-N
        pwb2d=np.zeros((len(a), rdl))
        for k in range(len(a)):
            pwb2d[k]=moving_average.data_points_running_average(a[k], N)
        pwb2d=pwb2d.transpose()

        # plot number of data points in each bin, onece for yCa (or yRa or ypw)
        # points at given frequency xfr[j]. The data points number is normalized
        # by maximum at each frequency
        for j in range(len(xfr[0:rdl])):
            xx=[xfr[j]]*ybin

            CXbmax=max(CXb2d[j])
            if CXbmax > 0:
                sc[0]=axes[kb].scatter(xx,yCX,c=CXb2d[j]/CXbmax,cmap=colormp,s=0.1, \
                    marker='s')

            Rabmax=max(Rab2d[j])
            if Rabmax > 0:
                sc[1]=axes[1+kb].scatter(xx,yRa,c=Rab2d[j]/Rabmax,cmap=colormp,s=0.1, \
                    marker='s')

            pwbmax=max(pwb2d[j])
            if pwbmax > 0:
                sc[2]=axes[2+kb].scatter(xx,ypw,c=pwb2d[j]/pwbmax,cmap=colormp, \
                    s=0.1,marker='s')

        ###############################
        #comparison data
        #La=40.0
        #LaChe=10.0
        #CXRPI=[645,720.0] # RPI Ca for 125 m branch
        #fRPI=[32.6,34.0]
        #CXRPI=[430,480]
        #RaRPI=[250e-3,500e-3]
        #PaRPI=[5.1,5.5]

        #fChe=[25.0,25.0]
        #CXChe=[169,187]
        #RaChe=[13.5,13.7]
        #if corx == 'Ca':
            #axes[kb].plot(freq, Cvac, ls='dashed', lw=0.3, color='red')
        #    axes[kb].yaxis.set_minor_locator(AutoMinorLocator())
        #    CXRPI[0]=3.0/2.0*CXRPI[0]*La/125.0*3.0/2.0
        #    CXRPI[1]=3.0/2.0*CXRPI[1]*La/125.0*3.0/2.0
        #    CXChe[0]=2.0*CXChe[0]*La/LaChe*3.0/2.0/5.0
        #    CXChe[1]=2.0*CXChe[1]*La/LaChe*3.0/2.0/5.0
        #else:
        #    CXRPI[0]=1.0e6/(pi2*fRPI[0]*3.0/2.0*CXRPI[0]*La/125.0*3.0/2.0)
        #    CXRPI[1]=1.0e6/(pi2*fRPI[1]*3.0/2.0*CXRPI[1]*La/125.0*3.0/2.0)
        #    CXChe[0]=1.0e6/(pi2*fChe[0]*2.0*CXChe[0]*La/LaChe*3.0/2.0)*4.0
        #    CXChe[1]=1.0e6/(pi2*fChe[1]*2.0*CXChe[1]*La/LaChe*3.0/2.0)*4.0

        #if comb=='Yes' and V0==8080 or V0==9696:
        ff=[2.7,33.0]
        if inpl == 1:
            XX=[1.7e5,1.7e5*(ff[0]/ff[1])**0.91]
        else:
            XX=[1.7e5,2.5e4]

        axes[0].plot(ff, XX, ls='dashed', lw=0.5, color='black', label='Fitting')
            
        RR=[3.0e4,3.0e4*(ff[0]/ff[1])**1.73]
        axes[1].plot(ff, RR, ls='dashed', lw=0.5, color='black', label='Fitting')

        #----- set axis properties for Ca or Xa------
        #axes[kb].scatter(fres[i], CXth[i], s=0.04, marker='+',color='black')
        #axes[kb].contour(xfr, yCX, CXb2d.transpose(), cmap=colormp)
        #axes[kb].scatter(fRPI,CXRPI, s=0.04, marker='s', color='red', \
        #    label='IMAGE RPI scaled')
        #axes[kb].scatter(fChe,CXChe, s=0.04, marker='s', color='black', \
        #    label='Chevalier scaled')
        axes[kb].set_xlim(fmin,fmax)
        axes[kb].set_xscale(xscale)
        axes[kb].set_ylim(minCX,maxCX)
        axes[kb].set_yscale(yscale)
        axes[kb].tick_params(axis='both',which='major',length=1.2,width=0.4, \
            pad=1,labelsize=2.5,left=True,right=True,bottom=True,top=True, \
            direction='in')
        axes[kb].tick_params(axis='both',which='minor',length=0.7,width=0.4, \
            pad=1,labelsize=0,left=True,right=True,bottom=True,top=True, \
            direction='in')
        axes[kb].set_xticklabels([])

        #----- set axis properties for Ra ------
        #note: / operator cannot be applied to list but can be used for numpy array
        #axes[1+kb].scatter(fres[i], Rath[i], s=0.04, marker='+',color='black')
        #axes[1+kb].contour(xfr[0:rdl], yRa, np.log10(Rab2d.transpose()), cmap=colormp)
        #axes[1+kb].scatter(fRPI,RaRPI, s=0.04, marker='s', color='red', \
        #    label='IMAGE RPI')
        #axes[1+kb].scatter(fChe,RaChe, s=0.04, marker='s', color='black', \
        #    label='Chevalier et al')
        axes[1+kb].set_xlim(fmin,fmax)
        axes[1+kb].set_xscale(xscale)
        axes[1+kb].set_yscale('log')
        axes[1+kb].set_ylim(minRa,maxRa)
        axes[1+kb].tick_params(axis='both',which='major',length=1.2,width=0.4, \
            pad=1,labelsize=2.5,left=True,right=True,bottom=True,top=True, \
            direction='in')
        axes[1+kb].tick_params(axis='both',which='minor',length=0.7,width=0.4, \
            pad=1,labelsize=0,left=True,right=True,bottom=True,top=True, \
            direction='in')
        axes[1+kb].set_xticklabels([])

        #----- set axis properties for pwr ------
        #axes[2+kb].scatter(fres[i], pwr[i], s=0.04, marker='+',color='black')
        #sc=axes[2+kb].contour(xfr, ypw, pwb2d.transpose(), cmap=colormp)
        #sc=axes[2+kb].scatter(fbin[i], pwb[i], c=bcnt[i], cmap=colormp, s=0.07, \
        # marker='s')
        #axes[2+kb].scatter(fRPI,PaRPI, s=0.04, marker='s', color='red', \
        #    label='IMAGE RPI')
        axes[2+kb].set_xlim(fmin,fmax)
        axes[2+kb].set_xscale(xscale)
        axes[2+kb].set_yscale('log')
        #axes[2+kb].set_yscale('linear')

        if bname == 'All-Bands':
            axes[2+kb].set_xticks([2.5,5,10,20,30,40])
            axes[2+kb].set_xticklabels(['2.5','5','10','20','30','40'])
        #axes[2+kb].set_ylim(minPr,maxPr)
        axes[2+kb].tick_params(axis='both',which='major',length=1.2,width=0.4, \
            pad=1,labelsize=2.5,left=True,right=True,bottom=True,top=True, \
            direction='in')
        axes[2+kb].tick_params(axis='both',which='minor',length=0.7,width=0.4, \
            pad=1,labelsize=0,left=True,right=True,bottom=True,top=True, \
            direction='in')
        axes[2+kb].set_xlabel('Resonance Frequency (kHz)',fontsize=2.5, \
            labelpad=0.7)

    # ---- vertical color bar ----
    #cbar=fig.colorbar(sc[0], ax=axes[3], shrink=0.9, pad=0.03, aspect=10)
    #cbar.ax.tick_params(labelsize=1.9, length=0.85, width=0.4, pad=0.4)
    #cbar.ax.set_ylabel('Noumber of Data Points', fontsize=2.1, labelpad=1.1)
    #cbar.outline.set_visible(False)

    # ---- vertical color bar ----
    #cbar=fig.colorbar(sc[1], ax=axes[1], shrink=0.9, pad=0.03, aspect=10)
    #cbar.ax.tick_params(labelsize=1.9, length=0.85, width=0.4, pad=0.4)
    #cbar.ax.set_ylabel('Noumber of Data Points', fontsize=2.1, labelpad=1.1)
    #cbar.outline.set_visible(False)

    # ---- vertical color bar ----
    #cbar=fig.colorbar(sc[2], ax=axes[5], shrink=0.9, pad=0.03, aspect=10)
    #cbar.ax.tick_params(labelsize=1.9, length=0.85, width=0.4, pad=0.4)
    #cbar.ax.set_ylabel('Noumber of Data Points', fontsize=2.1, labelpad=1.1)
    #cbar.outline.set_visible(False)

    # --- y-axis label
    if corx == 'Ca':
        axes[0].set_ylabel('$C_a$ (pF)',fontsize=2.5,labelpad=0.7)
    else:
        axes[0].set_ylabel('$-X_a$ (Ohm)',fontsize=2.5,labelpad=0.7)
    axes[1].set_ylabel('$R_a$ (Ohm)',fontsize=2.5,labelpad=0.7)
    #axes[2].set_ylabel('$P_{rad}$ (V)',fontsize=2.5,labelpad=0.7)
    axes[2].set_ylabel('$V_{a}$ (V)',fontsize=3,labelpad=0.7)

    # --- add text: date, inductance band, Y+ / Y - 1st harmonic ----
    datemax=max(date)
    datemin=min(date)
    if inpflag == 'In':
        txt=sL1+' Inside Nominal Plasmasphere'
    elif inpflag == 'Out':
        txt=sL1+' Outside Plasmasphere'
    else:
        txt=sL1+' All Regions'

    if comb == 'No':
        # make y-axis tick lables invisible for right panels
        for j in range(3,6):
            axes[j].set_yticklabels([])

        xloc=1.0
        axes[0].text(0,1.04,'Y+ 1st Harmonic',fontsize=2.3,ha='left',va='bottom', \
        transform=axes[0].transAxes)
        axes[3].text(1,1.04,'Y- 1st Harmonic',fontsize=2.3,ha='right',va='bottom', \
        transform=axes[3].transAxes)
    else:
        xloc=0.5
        #for j in range(3):
        #    axes[j].legend(loc='upper center',fontsize=1.6,bbox_to_anchor=(0.65,0.99), \
        #    frameon=False)

    axes[0].text(xloc,1.12,datemin[0:10]+'$-$'+datemax[0:10],fontsize=2.5, \
        ha='center',va='bottom',transform=axes[0].transAxes)
    axes[0].text(xloc,1.02,txt,fontsize=2.5,ha='center',va='bottom', \
        transform=axes[0].transAxes)

    # ---- color bar ----
    if comb == 'Yes':
        xloc=0.085
    else:
        xloc=0.6
    axins = inset_axes(axes[2], width='70%', height='8%', loc='lower left', \
    bbox_to_anchor=(xloc,-0.46,1,1), bbox_transform=axes[2].transAxes)
    cbar=fig.colorbar(sc[2], cax=axins, shrink=0.9, pad=0.04, aspect=10, \
        fraction=0.1, orientation='horizontal',boundaries=None)
    cbar.ax.tick_params(labelsize=2.3, length=1.3, width=0.3, pad=0.4)
    cbar.ax.set_xlabel('Relative Number of Data Points', fontsize=2.3, labelpad=1.1)
    cbar.outline.set_visible(False)

    return canv, fig, toolbar
